local Version = 1.10

--------------------------------------------------------------------------------------------
-- generic helper functions --

function deepcopy(object)
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        end
        local new_table = {}
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

--------------------------------------------------------------------------------------------
-- helper functions for GUI --

local NameCount = 0
local function GetUniqueName()
	NameCount = NameCount + 1
	return "Stats_"..NameCount
end

-- ColorBox --
local CurrentColorBox
local function ColorChanged(oldValues)
	if oldValues then 
		local r,g,b,a = unpack(oldValues)
		CurrentColorBox:SetBackdropColor(r,g,b,a)
	else
		local a = OpacitySliderFrame:GetValue()
		local r,g,b = ColorPickerFrame:GetColorRGB()
		CurrentColorBox:SetBackdropColor(r,g,b,1-a)
	end
end

local function ShowColorPicker(ColorBox)
	CurrentColorBox = ColorBox
	local r,g,b,a = ColorBox:GetBackdropColor()
	ColorPickerFrame.previousValues  = {r,g,b,a}
	ColorPickerFrame:SetColorRGB(r,g,b);
	ColorPickerFrame.hasOpacity = true
	ColorPickerFrame.opacity = 1-a
	ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = 	ColorChanged, ColorChanged, ColorChanged;
	ColorPickerFrame:Hide();
	ColorPickerFrame:Show();
end

local function CreateColorBox(parent, x, y, label)
	local colorbox = CreateFrame("Button", GetUniqueName(), parent)

	colorbox:SetPoint("TOPLEFT", x, y)
	colorbox:SetWidth(24)
	colorbox:SetHeight(24)
	colorbox:SetBackdrop({bgFile = "Interface\\ChatFrame\\ChatFrameColorSwatch", edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = false, tileSize = 16, edgeSize = 8, insets = { left = 1, right = 1, top = 1, bottom = 1 }})
	colorbox:SetScript("OnClick", function() ShowColorPicker(colorbox) end)

	if label ~= nil then
		colorbox.Label = colorbox:CreateFontString(nil, 'ARTWORK', 'GameFontWhiteSmall')
		colorbox.Label:SetPoint("TOPLEFT", colorbox, "TOPRIGHT", 4, -7)
		colorbox.Label:SetText(label)
	end

	colorbox.SetColor =
		function(self, c)
			colorbox:SetBackdropColor(c[1], c[2], c[3], c[4])
		end

	colorbox.GetColor =
		function()
			local r,g,b,a = colorbox:GetBackdropColor()
			return {r,g,b,a}
		end

	return colorbox
end

-- EditBox --
local function CreateEditBox(parent, x, y, w, label)
	local editbox = CreateFrame("EditBox", GetUniqueName(), parent, "InputBoxTemplate")
	editbox:SetPoint("TOPLEFT", x+6, y)
	editbox:SetAutoFocus(false)
	editbox:SetTextInsets(0, 0, 3, 3)
	editbox:SetMaxLetters(8)
	editbox:SetWidth(w)
	editbox:SetHeight(24)

	if label ~= nil then
		editbox.Label = editbox:CreateFontString(nil, 'ARTWORK', 'GameFontWhiteSmall')
		editbox.Label:SetPoint("TOPLEFT", editbox, "TOPRIGHT", 4, -7)
		editbox.Label:SetText(label)
	end

	return editbox
end

-- CheckButton --
local function CreateCheckButton(parent, x, y, label)
	local checkbutton = CreateFrame("CheckButton", GetUniqueName(), parent, "InterfaceOptionsCheckButtonTemplate" )
	checkbutton:SetPoint("TOPLEFT", x, y)
	checkbutton:SetWidth(24)
	checkbutton:SetHeight(24)
	_G[checkbutton:GetName() .. "Text"]:SetText(label)
	return checkbutton
end

-- Label --
local function CreateLabel(parent, x, y, labeltext, color)
	local label = parent:CreateFontString(GetUniqueName(), 'ARTWORK', 'GameFontWhiteSmall')
	label:SetPoint("TOPLEFT", x, y)
	label:SetText(labeltext)
	if color ~= nil then
		label:SetTextColor(color[1], color[2], color[3], color[4])
	end
	return label
end

-- GroupFrame --
local function CreateGroupFrame(parent, x, y, w, h, color)
	local group = CreateFrame("Frame", nil, parent)
	group:SetPoint("TOPLEFT", x, y)
	group:SetWidth(w)
	group:SetHeight(h)
	group:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 12, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
	if color ~= nil then
		group:SetBackdropColor(color[1], color[2], color[3], color[4])
		group:SetBackdropBorderColor(color[1], color[2], color[3], color[4])
	else
		group:SetBackdropColor(0.2, 0.2, 0.2, 1)
		group:SetBackdropBorderColor(0.2, 0.2, 0.2, 1)
	end
	return group
end

--------------------------------------------------------------------------------------------

--StatsConfig				-- saved global

local StatFrames = {}		-- actual HUD elements
local OptionsPanel = {}		-- options panel for blizzard's interface menu
local Spec = 1			-- current talent group

local StatInfo =			-- static stats data
{
	-- GENERAL
	{
		Name = "Average Item Level", Prefix="iLvl: ", Format = "%d",
		GetValue = function ()
			return GetAverageItemLevel()
		end
	},


	-- ATTRIBUTES
	{
		Name = "Strength", Prefix="Str: ", Format = "%d",
		GetValue = function ()
			_,stat = UnitStat("player", 1)
			return stat
		end
	},
	{
		Name = "Agility", Prefix="Agi: ", Format = "%d",
		GetValue = function ()
			_,stat = UnitStat("player", 2)
			return stat
		end
	},
	{
		Name = "Stamina", Prefix="Sta: ", Format = "%d",
		GetValue = function ()
			_,stat = UnitStat("player", 3)
			return stat
		end
	},
	{
		Name = "Intellect", Prefix="Int: ", Format = "%d",
		GetValue = function ()
			_,stat = UnitStat("player", 4)
			return stat
		end
	},
	{
		Name = "Spirit", Prefix="Spi: ", Format = "%d",
		GetValue = function ()
			_,stat = UnitStat("player", 5)
			return stat
		end
	},


	-- MASTERY
	{
		Name = "Mastery", Prefix="Mastery: ", Format = "%.2f",
		GetValue = function ()
			return GetMastery()
		end
	},


	-- MELEE
	{
		Name = "Melee Attack Power", Prefix="AP: ", Format = "%d",
		GetValue = function ()
			local base, posBuff, negBuff = UnitAttackPower("player");
			return base + posBuff + negBuff;
		end
	},
	{
		Name = "Melee Crit Chance", Prefix="Crit: ", Format = "%.2f",
		GetValue = function ()
			return GetCritChance()
		end
	},
	{
		Name = "Melee Haste", Prefix="Haste: ", Format = "%.2f",
		GetValue = function ()
			return GetMeleeHaste()
		end
	},
	{
		Name = "Melee Hit Chance", Prefix="Hit: ", Format = "%.2f",
		GetValue = function ()
			return GetCombatRatingBonus(CR_HIT_MELEE) + GetHitModifier()
		end
	},
	{
		Name = "Expertise", Prefix="Exp: ", Format = "%d",
		GetValue = function ()
			return GetExpertise()
		end
	},


	-- RANGED
	{
		Name = "Ranged Attack Power", Prefix="AP: ", Format = "%d",
		GetValue = function ()
			local base, posBuff, negBuff = UnitRangedAttackPower("player");
			return base + posBuff + negBuff;
		end
	},
	{
		Name = "Ranged Crit Chance", Prefix="Crit: ", Format = "%.2f",
		GetValue = function ()
			return GetRangedCritChance()
		end
	},
	{
		Name = "Ranged Haste", Prefix="Haste: ", Format = "%.2f",
		GetValue = function ()
			return GetRangedHaste()
		end
	},
	{
		Name = "Ranged Hit Chance", Prefix="Hit: ", Format = "%.2f",
		GetValue = function ()
			return GetCombatRatingBonus(CR_HIT_RANGED) + GetHitModifier()
		end
	},


	-- SPELL
	{
		Name = "Spell Damage", Prefix="SD: ", Format = "%d",
		GetValue = function ()
			return GetSpellBonusDamage(2)
		end
	},
	{
		Name = "Spell Healing", Prefix="SH: ", Format = "%d",
		GetValue = function ()
			return GetSpellBonusHealing()
		end
	},
	{
		Name = "Spell Crit Chance", Prefix="Crit: ", Format = "%.2f",
		GetValue = function ()
			return GetSpellCritChance(2)
		end
	},
	{
		Name = "Spell Haste", Prefix="Haste: ", Format = "%.2f",
		GetValue = function ()
			return UnitSpellHaste("player")
		end
	},
	{
		Name = "Spell Hit Chance", Prefix="Hit: ", Format = "%.2f",
		GetValue = function ()
			return GetCombatRatingBonus(CR_HIT_SPELL) + GetSpellHitModifier()
		end
	},
	{
		Name = "Spell Penetration", Prefix="SPen: ", Format = "%d",
		GetValue = function ()
			return GetSpellPenetration()
		end
	},
	{
		Name = "Mana Regen", Prefix="MP5: ", Format = "%d",
		GetValue = function ()
			base, casting = GetManaRegen()
			return base*5
		end
	},
	{
		Name = "Combat Regen", Prefix="CMP5: ", Format = "%d",
		GetValue = function ()
			base, casting = GetManaRegen()
			return casting*5
		end
	},


	-- DEFENSE
	{
		Name = "Armor", Prefix="Armor: ", Format = "%d",
		GetValue = function ()
			local baseArmor , effectiveArmor, armor, posBuff, negBuff = UnitArmor("player");
			return effectiveArmor
		end
	},
	{
		Name = "Dodge Chance", Prefix="Dodge: ", Format = "%.2f",
		GetValue = function ()
			return GetDodgeChance()
		end
	},
	{
		Name = "Parry Chance", Prefix="Parry: ", Format = "%.2f",
		GetValue = function ()
			return GetParryChance()
		end
	},
	{
		Name = "Block Chance", Prefix="Block: ", Format = "%.2f",
		GetValue = function ()
			return GetBlockChance()
		end
	},
	{
		Name = "Resilience", Prefix="Resi: ", Format = "%d",
		GetValue = function ()
			return GetCombatRating(COMBAT_RATING_RESILIENCE_PLAYER_DAMAGE_TAKEN)
		end
	},


	-- RESISTANCE
	{
		Name = "Arcane Resistance", Prefix="Arcane: ", Format = "%d",
		GetValue = function ()
			_,res = UnitResistance("player", 6)
			return res
		end
	},
	{
		Name = "Fire Resistance", Prefix="Fire: ", Format = "%d",
		GetValue = function ()
			_,res = UnitResistance("player", 2)
			return res
		end
	},
	{
		Name = "Nature Resistance", Prefix="Nature: ", Format = "%d",
		GetValue = function ()
			_,res = UnitResistance("player", 3)
			return res
		end
	},
	{
		Name = "Frost Resistance", Prefix="Frost: ", Format = "%d",
		GetValue = function ()
			_,res = UnitResistance("player", 4)
			return res
		end
	},
	{
		Name = "Shadow Resistance", Prefix="Shadow: ", Format = "%d",
		GetValue = function ()
			_,res = UnitResistance("player", 5)
			return res
		end
	}
}

--------------------------------------------------------------------------------------------

-- categories for interface options
local StatCategory =
{
	{ Name = "Item Level", Items = 1 },
	{ Name = "Attributes", Items = 5 },
	{ Name = "Mastery", Items = 1 },
	{ Name = "Melee", Items = 5 },
	{ Name = "Ranged", Items = 4 },
	{ Name = "Spell", Items = 8 },
	{ Name = "Defense", Items = 5 },
	{ Name = "Resistance", Items = 5 }
}

--------------------------------------------------------------------------------------------

local function CreateConfigStats(stats)
	for i=1,#(StatInfo) do
		stats[i] =
		{
			Enabled = false,
			PosX = 0,
			PosY = #(StatInfo)*11 - i*22,
			TextColor = { 1, 1, 1, 1 },
			Width = 92,
			FontSize = 14,
			Prefix = StatInfo[i].Prefix
		}
	end
end

local function CreateConfig()
	print("Stats: creating default config...")
	StatsConfig = {}
	StatsConfig.Version = Version
	StatsConfig.LockFrames = false
	StatsConfig.BgColor = { 0, 0, 0, 0.5 }
	StatsConfig.Decimals = 2
	StatsConfig.Stats = {}
	StatsConfig.Stats[1] = {}
	StatsConfig.Stats[2] = {}
	CreateConfigStats(StatsConfig.Stats[1])
	CreateConfigStats(StatsConfig.Stats[2])
end



local function UpgradeConfig_100_101()
	print("Stats: upgrading config... 1.0.0 -> 1.0.1")
	StatsConfig.Version = 1.01

	local function CreateStatTable(i,y)
		return 
		{
			Enabled = false,
			PosX = 0,
			PosY = y,
			TextColor = { 1, 1, 1, 1 },
			Width = 92,
			FontSize = 14,
			Prefix = StatInfo[i].Prefix
		}
	end

	tinsert(StatsConfig.Stats, 4, CreateStatTable(4, 3*11 - 1*22))
	tinsert(StatsConfig.Stats, 8, CreateStatTable(8, 3*11 - 2*22))
	tinsert(StatsConfig.Stats, 13, CreateStatTable(13, 3*11 - 3*22))
end

local function UpgradeConfig_101_102()
	print("Stats: upgrading config... 1.0.1 -> 1.0.2")
	StatsConfig.Version = 1.02

	local function CreateStatTable(i,y)
		return 
		{
			Enabled = false,
			PosX = 0,
			PosY = y,
			TextColor = { 1, 1, 1, 1 },
			Width = 92,
			FontSize = 14,
			Prefix = StatInfo[i].Prefix
		}
	end

	tinsert(StatsConfig.Stats, 14, CreateStatTable(14, 2*11 - 1*22))
	tinsert(StatsConfig.Stats, 15, CreateStatTable(15, 2*11 - 2*22))
end

local function UpgradeConfig_102_103()
	print("Stats: upgrading config... 1.0.2 -> 1.0.3")
	StatsConfig.Version = 1.03

	local function CreateStatTable(i,y)
		return 
		{
			Enabled = false,
			PosX = 0,
			PosY = y,
			TextColor = { 1, 1, 1, 1 },
			Width = 92,
			FontSize = 14,
			Prefix = StatInfo[i].Prefix
		}
	end

	local y = 15*22/2	-- numNewStats * frameHeight /2

	for i=1,7 do
		tinsert(StatsConfig.Stats, i, CreateStatTable(i, y))
		y = y - 22
	end

	i = 12
	tinsert(StatsConfig.Stats, i, CreateStatTable(i, y))
	y = y - 22

	i = 22
	tinsert(StatsConfig.Stats, i, CreateStatTable(i, y))
	y = y - 22

	for i=29,34 do
		tinsert(StatsConfig.Stats, i, CreateStatTable(i, y))
		y = y - 22
	end
end

local function UpgradeConfig_103_104()
	print("Stats: upgrading config... 1.0.3 -> 1.0.4")
	StatsConfig.Version = 1.04

	StatsConfig.Decimals = 2
end

local function UpgradeConfig_104_110()
	print("Stats: upgrading config... 1.0.4 -> 1.1.0")
	StatsConfig.Version = 1.10

	local tmp = StatsConfig.Stats
	StatsConfig.Stats = {}
	StatsConfig.Stats[1] = deepcopy(tmp)
	StatsConfig.Stats[2] = deepcopy(tmp)
end

--------------------------------------------------------------------------------------------

-- mouse drag functions for HUD elments
local function OnMouseDown(self,button)
	if button=="LeftButton" then
		_,_,_,self.XO,self.YO = self:GetPoint()			-- original position in parent space
		self:StartMoving()								-- de-anchors from parent
		_,_,_,self.XOS,self.YOS = self:GetPoint()		-- original position in screen space
		self.Moving = true
	end
end

local function OnMouseUp(self,button)
	if self.Moving then
		self.Moving = false
		local _,_,_,XNS,YNS = self:GetPoint()			-- new position in screen space
		self:StopMovingOrSizing()						-- anchors to an arbitrary point
		self:SetUserPlaced(false)
		self:ClearAllPoints()
		local XN = XNS - self.XOS + self.XO				-- new position in the correct parent space
		local YN = YNS - self.YOS + self.YO
		self:SetPoint("CENTER", XN, YN)					-- re-anchor to the correct parent and point
		StatsConfig.Stats[Spec][self.Stat].PosX = XN
		StatsConfig.Stats[Spec][self.Stat].PosY = YN
	end
end

-- applies config data to a HUD element
local function UpdateStatFrame(frame)
	local stat = frame.Stat

	frame.Background:SetTexture(StatsConfig.BgColor[1], StatsConfig.BgColor[2], StatsConfig.BgColor[3], StatsConfig.BgColor[4])

	frame.StatText:SetFont("Fonts\\ARIALN.TTF", StatsConfig.Stats[Spec][stat].FontSize)
	frame.StatText:SetTextColor(StatsConfig.Stats[Spec][stat].TextColor[1], StatsConfig.Stats[Spec][stat].TextColor[2], StatsConfig.Stats[Spec][stat].TextColor[3], StatsConfig.Stats[Spec][stat].TextColor[4])
	frame.StatText:SetText("A")		-- need for GetStringHeight() to set the height of the frame

	frame:SetWidth(StatsConfig.Stats[Spec][stat].Width)
	frame:SetHeight(frame.StatText:GetStringHeight() * 1.5)

	frame:EnableMouse(not StatsConfig.LockFrames)

	frame:SetPoint("CENTER", StatsConfig.Stats[Spec][stat].PosX, StatsConfig.Stats[Spec][stat].PosY)

	frame:Show()
end

-- creates a HUD element
local function CreateStatFrame(stat)
	local frame = CreateFrame("Frame",nil,UIParent)
	frame.Stat = stat

	frame.Background = frame:CreateTexture(nil,"BACKGROUND")
	frame.Background:SetAllPoints(frame)

	frame.StatText = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	frame.StatText:SetAllPoints(frame)

	frame:SetFrameStrata("BACKGROUND")
	frame:SetPoint("CENTER", StatsConfig.Stats[Spec][stat].PosX, StatsConfig.Stats[Spec][stat].PosY)
	frame:SetMovable(true)
	frame:SetScript("OnMouseDown", OnMouseDown)
	frame:SetScript("OnMouseUp", OnMouseUp)

	UpdateStatFrame(frame)

	return frame
end

--------------------------------------------------------------------------------------------

-- applies config data to all HUD elements (lazy creates frames)
local function UpdateStatFrames()
	for i=1,#(StatInfo) do
		if StatsConfig.Stats[Spec][i].Enabled then
			if StatFrames[i] == nil then
				StatFrames[i] = CreateStatFrame(i)
			else
				UpdateStatFrame(StatFrames[i])
			end
		else
			if StatFrames[i] ~= nil then
				StatFrames[i]:Hide()
			end
		end
	end
end

--------------------------------------------------------------------------------------------

local nextTime = GetTime()

-- updates values in HUD elements 10 times per second
local function OnUpdate(self, elapsed)
	if GetTime() < nextTime then
		return
	end
	nextTime = nextTime + 0.1

	for i=1,#(StatInfo) do
		if StatsConfig.Stats[Spec][i].Enabled then
			local stat = format(StatInfo[i].Format, StatInfo[i].GetValue())
			if StatInfo[i].Format == "%.2f" then
				stat = format("%."..StatsConfig.Decimals.."f", StatInfo[i].GetValue())
			end
			StatFrames[i].StatText:SetText(StatsConfig.Stats[Spec][i].Prefix..stat)
		end
	end
end

--------------------------------------------------------------------------------------------

local function Okay()
	StatsConfig.LockFrames = OptionsPanel.Widgets.LockFrames:GetChecked()
	StatsConfig.BgColor = OptionsPanel.Widgets.BgColor:GetColor()
	StatsConfig.Decimals = OptionsPanel.Widgets.Decimals:GetNumber()
	for i=1,#(StatInfo) do
		StatsConfig.Stats[Spec][i].Enabled = OptionsPanel.Widgets.Stats[i].Enabled:GetChecked()
		StatsConfig.Stats[Spec][i].Prefix = OptionsPanel.Widgets.Stats[i].Prefix:GetText()
		StatsConfig.Stats[Spec][i].Width = OptionsPanel.Widgets.Stats[i].Width:GetNumber()
		StatsConfig.Stats[Spec][i].FontSize = OptionsPanel.Widgets.Stats[i].FontSize:GetNumber()
		StatsConfig.Stats[Spec][i].TextColor = OptionsPanel.Widgets.Stats[i].TextColor:GetColor()
	end

	if StatsConfig.Decimals < 0 then
		StatsConfig.Decimals = 0
	end
	if StatsConfig.Decimals > 5 then
		StatsConfig.Decimals = 5
	end

	UpdateStatFrames()
end

local function Cancel()
end

local function Default()
	CreateConfigStats(StatsConfig.Stats[Spec])
end

local function Refresh()
	OptionsPanel.Widgets.LockFrames:SetChecked(StatsConfig.LockFrames)
	OptionsPanel.Widgets.BgColor:SetColor(StatsConfig.BgColor)
	OptionsPanel.Widgets.Decimals:SetNumeric(true)
	OptionsPanel.Widgets.Decimals:SetNumber(StatsConfig.Decimals)
	OptionsPanel.Widgets.Decimals:SetCursorPosition(0)
	for i=1,#(StatInfo) do
		OptionsPanel.Widgets.Stats[i].Enabled:SetChecked(StatsConfig.Stats[Spec][i].Enabled)
		OptionsPanel.Widgets.Stats[i].Prefix:SetText(StatsConfig.Stats[Spec][i].Prefix)
		OptionsPanel.Widgets.Stats[i].Prefix:SetCursorPosition(0)
		OptionsPanel.Widgets.Stats[i].Width:SetNumeric(true)
		OptionsPanel.Widgets.Stats[i].Width:SetNumber(StatsConfig.Stats[Spec][i].Width)
		OptionsPanel.Widgets.Stats[i].Width:SetCursorPosition(0)
		OptionsPanel.Widgets.Stats[i].FontSize:SetNumeric(true)
		OptionsPanel.Widgets.Stats[i].FontSize:SetNumber(StatsConfig.Stats[Spec][i].FontSize)
		OptionsPanel.Widgets.Stats[i].FontSize:SetCursorPosition(0)
		OptionsPanel.Widgets.Stats[i].TextColor:SetColor(StatsConfig.Stats[Spec][i].TextColor)
	end
end

local function CreateOptionPanel()
	OptionsPanel = CreateFrame("Frame", nil, UIParent)
	OptionsPanel.name = "Stats"
	OptionsPanel.okay = Okay
	OptionsPanel.cancel = Cancel
	OptionsPanel.default = Default
	OptionsPanel.refresh = Refresh
	OptionsPanel.Widgets = {}

	Panel = CreateFrame("Frame", nil, ScrollFrame)
	Panel:SetWidth(375)
	Panel:SetHeight(410)

	local ScrollFrame = CreateFrame("ScrollFrame", "StatsOptionsScrollFrame", OptionsPanel, "FauxScrollFrameTemplate")
	ScrollFrame:SetPoint("TOPLEFT", 10, -10)
	ScrollFrame:SetPoint("BOTTOMRIGHT", -30, 10)
	ScrollFrame:SetScrollChild(Panel)
	FauxScrollFrame_Update(StatsOptionsScrollFrame, 170 + #(StatCategory)*40 + #(StatInfo)*90, 410, 1)

	local catGroup = CreateGroupFrame(Panel, 10, -10, 360, 40 + 90)
	CreateLabel(catGroup, 20, -10, "Global Settings", {1, 1, 0.5, 1})
	local group = CreateGroupFrame(catGroup, 10, -30, 360-20, 90, {0.3, 0.3, 0.3, 1})
	OptionsPanel.Widgets.LockFrames = CreateCheckButton(group, 20, -20, "Lock Frames")
	OptionsPanel.Widgets.BgColor = CreateColorBox(group, 190, -20, "Backdrop Color")
	OptionsPanel.Widgets.Decimals = CreateEditBox(group, 20, -50, 32, "Number of Decimals")

	OptionsPanel.Widgets.Stats = {}
	local y = -170

	local i = 1;
	for c=1,#(StatCategory) do
		local catGroup = CreateGroupFrame(Panel, 10, y, 360, 30 + 90*StatCategory[c].Items)
		CreateLabel(catGroup, 20, -10, StatCategory[c].Name, {1, 1, 0.5, 1})
		local y2 = -30
		for j=1,StatCategory[c].Items do
			local group = CreateGroupFrame(catGroup, 10, y2, 360-20, 80, {0.3, 0.3, 0.3, 1})
			OptionsPanel.Widgets.Stats[i] = {}
			OptionsPanel.Widgets.Stats[i].Enabled = CreateCheckButton(group, 10, -10, "Enable "..StatInfo[i].Name)
			OptionsPanel.Widgets.Stats[i].Prefix = CreateEditBox(group, 225, -10, 58, "Prefix")
			OptionsPanel.Widgets.Stats[i].Width = CreateEditBox(group, 10, -40, 32, "Width")
			OptionsPanel.Widgets.Stats[i].FontSize = CreateEditBox(group, 115, -40, 32, "Font Size")
			OptionsPanel.Widgets.Stats[i].TextColor = CreateColorBox(group, 225, -40, "Text Color")
			i = i + 1
			y2 = y2 - 90
		end
		y = y - StatCategory[c].Items * 90 - 40
	end

	InterfaceOptions_AddCategory(OptionsPanel)
end

--------------------------------------------------------------------------------------------

-- first time dialog
local NewConfig = false
StaticPopupDialogs["STATS_NEW_CONFIG"] = {
	text = "Stats hasn't been configured for this character. Would you like to visit the interface options now?",
	button1 = "ok",
	button2 = "later",
	timeout = 0,
	OnAccept = function() InterfaceOptionsFrame_OpenToCategory(OptionsPanel) end,
}

--------------------------------------------------------------------------------------------


local MainFrame = CreateFrame("Frame",nil,UIParent)

local function OnEvent(self, event, arg1)
	if event == "ADDON_LOADED" and arg1 == "Stats" then
		if StatsConfig == nil then
			CreateConfig()
			NewConfig = true
		end
		if StatsConfig.Version == 1 then
			UpgradeConfig_100_101()
		end
		if StatsConfig.Version == 1.01 then
			UpgradeConfig_101_102()
		end
		if StatsConfig.Version == 1.02 then
			UpgradeConfig_102_103()
		end
		if StatsConfig.Version == 1.03 then
			UpgradeConfig_103_104()
		end
		if StatsConfig.Version == 1.04 then
			UpgradeConfig_104_110()
		end

	elseif event == "PLAYER_LOGIN" then
		Spec = GetActiveTalentGroup(false, false)
		CreateOptionPanel()
		UpdateStatFrames()
		MainFrame:SetScript("OnUpdate", OnUpdate)
		MainFrame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
		if NewConfig then
			StaticPopup_Show("STATS_NEW_CONFIG")
		end

	elseif event == "ACTIVE_TALENT_GROUP_CHANGED" then
		Spec = GetActiveTalentGroup(false, false)
		Refresh()
		UpdateStatFrames()
	end
end

MainFrame:RegisterEvent("ADDON_LOADED");
MainFrame:RegisterEvent("PLAYER_LOGIN");

MainFrame:SetScript("OnEvent", OnEvent);
